// Timer Interrupt handler


#include "include/types.h"
#include "include/param.h"
#include "include/riscv.h"
#include "include/sbi.h"
#include "include/spinlock.h"
#include "include/timer.h"
#include "include/times.h"
#include "include/printf.h"
#include "include/proc.h"
#include "include/syscall.h"
#include "include/defs.h"

struct spinlock tickslock;
uint ticks;

void timerinit() {
    initlock(&tickslock, "time");
    #ifdef DEBUG
    printf("timerinit\n");
    #endif
}

void
set_next_timeout() {
    // There is a very strange bug,
    // if comment the `printf` line below
    // the timer will not work.

    // this bug seems to disappear automatically
    // printf("");
    sbi_set_timer(r_time() + INTERVAL);
}

void timer_tick() {
    acquire(&tickslock);
    ticks++;
    wakeup(&ticks);
    release(&tickslock);
    set_next_timeout();
}

uint64 sys_times(void){
    uint64 addr;

    if(argaddr(0, &addr)<0){
        return -1;
    }
    
    struct tms *tbuf = (struct tms *)addr;
    struct proc *p = myproc();

    if (addr){
        if(copyout(p->pagetable, (uint64)&(tbuf -> tms_utime), (char *)&(p->utime), sizeof(p->utime)) < 0||
        copyout(p->pagetable, (uint64)&(tbuf -> tms_stime), (char *)&(p->stime), sizeof(p->stime)) < 0||
        copyout(p->pagetable, (uint64)&(tbuf -> tms_cutime), (char *)&(p->cutime), sizeof(p->cutime)) < 0||
        copyout(p->pagetable, (uint64)&(tbuf -> tms_cstime), (char *)&(p->cstime), sizeof(p->cstime)) < 0){
            return -1;
        }
    }
    return ticks;
}

uint64 sys_gettimeofday(void){
    uint64 addr;

    if(argaddr(0, &addr)<0){
        return -1;
    }  

    struct timespec *tbuf = (struct timespec *)addr;
    struct proc *p = myproc();
    uint64 sec = ticks;
    uint64 usec = 0;

    if(addr){
        if(copyout(p->pagetable, (uint64)&(tbuf -> sec), (char *)&sec, sizeof(sec)) < 0||
        copyout(p->pagetable, (uint64)&(tbuf -> usec), (char *)&usec, sizeof(usec)) < 0){
            return -1;
        }
    }
    return 0;
}
